/*
%%% copyright-cmetz-98
This software is Copyright 1998 by Craig Metz, All Rights Reserved.
The Inner Net License Version 2 applies to this software.
You should have received a copy of the license with this software. If
you didn't get a copy, you may request one from <license@inner.net>.

*/

#ifndef _NET_SECURITY_H
#define _NET_SECURITY_H 1

#define NET_SECURITY_REQUEST_MAX 16
#define NET_SECURITY_OPERATION_MAX 16

#if defined(KERNEL) || defined(__KERNEL__)
#include <net/security_osdep.h>
#else /* defined(KERNEL) || defined(__KERNEL__) */
#ifndef AF_INET /* XXX */
#include <sys/socket.h>
#endif /* AF_INET */
#ifndef IPPROTO_TCP /* XXX */
#include <netinet/in.h>
#endif /* IPPROTO_TCP */
#endif /* defined(KERNEL) || defined(__KERNEL__) */

union net_security_sockaddr_union {
  struct sockaddr sa;
#if AF_INET
  struct sockaddr_in sin;
#endif /* AF_INET */
#if AF_INET6
  struct sockaddr_in6 sin6;
#endif /* AF_INET6 */
};

struct net_security_operation {
  uint8_t net_security_operation_type;
  uint8_t net_security_operation_preference;
  uint8_t net_security_operation_barrier;
  uint8_t net_security_operation_reserved;
  /* compiler-inserted pad if 64 bit */
  union {
    struct {
      uint8_t __alloctype;
      uint8_t __hints;
      uint8_t __algid;
    } __forsa;
    union net_security_sockaddr_union __sockaddr;
  } __union;
};

#define net_security_operation_alloctype __union.__forsa.__alloctype
#define net_security_operation_hints __union.__forsa.__hints
#define net_security_operation_algid __union.__forsa.__algid
#define net_security_operation_sockaddr __union.__sockaddr.sa

/* Operation types (net_security_operation_type): */
/*   Authenticate (A) */
#define NET_SECURITY_TYPE_AUTHENTICATE         1
/*   Encrypt (E) */
#define NET_SECURITY_TYPE_ENCRYPT              2
/*   Encapsulate (N) */
#define NET_SECURITY_TYPE_ENCAPSULATE          3

/* Operation preferences (net_security_operation_preference): */
/*   Default (d) */
#define NET_SECURITY_PREFERENCE_DEFAULT        0
/*   Use (u) */
#define NET_SECURITY_PREFERENCE_USE            1
/*   Require (r) */
#define NET_SECURITY_PREFERENCE_REQUIRE        2
/*   Never (n) */
#define NET_SECURITY_PREFERENCE_NEVER          3

/* Operation SA allocation types (net_security_operation_alloctype): */
/*   System */
#define NET_SECURITY_ALLOCTYPE_SYSTEM          1
/*   GID */
#define NET_SECURITY_ALLOCTYPE_GID             2
/*   UID */
#define NET_SECURITY_ALLOCTYPE_UID             3
/*   PGID */
#define NET_SECURITY_ALLOCTYPE_PGID            4
/*   Family */
#define NET_SECURITY_ALLOCTYPE_FAMILY          5
/*   Socket */
#define NET_SECURITY_ALLOCTYPE_SOCKET          6
/*   Maximum value */
#define NET_SECURITY_ALLOCTYPE_MAX             NET_SECURITY_ALLOCTYPE_SOCKET

/* Operation SA hints (net_security_operation_hints): */
/*   Mask for sensitivity hints */
#define NET_SECURITY_HINTS_SENSITIVITY          0x0f
/*   Mask for volume hints */
#define NET_SECURITY_HINTS_VOLUME               0x30
/*   Mask for latency hints */
#define NET_SECURITY_HINTS_LATENCY              0xc0

/*   Unknown sensitivity */
#define NET_SECURITY_SENSITIVITY_UNKNOWN        0x00
/*   Lowest sensitivity */
#define NET_SECURITY_SENSITIVITY_LOWEST         0x01
/*   Low sensitivity */
#define NET_SECURITY_SENSITIVITY_LOW            0x04
/*   Medium sensitivity */
#define NET_SECURITY_SENSITIVITY_MEDIUM         0x08
/*   High sensitivity */
#define NET_SECURITY_SENSITIVITY_HIGH           0x0c
/*   Highest sensitivity */
#define NET_SECURITY_SENSITIVITY_HIGHEST        0x0f
/*   Unknown volume */
#define NET_SECURITY_SENSITIVITY_UNKNOWN        0x00

/*   Unknown volume */
#define NET_SECURITY_VOLUME_UNKNOWN             0x00
/*   Low volume */
#define NET_SECURITY_VOLUME_LOW                 0x10
/*   Medium volume */
#define NET_SECURITY_VOLUME_MEDIUM              0x20
/*   High volume */
#define NET_SECURITY_VOLUME_HIGH                0x30

/*   Unknown latency */
#define NET_SECURITY_LATENCY_UNKNOWN            0x00
/*   Low latency */
#define NET_SECURITY_LATENCY_LOW                0x40
/*   Medium latency */
#define NET_SECURITY_LATENCY_MEDIUM             0x80
/*   High latency */
#define NET_SECURITY_LATENCY_HIGH               0xc0

/* Identity types: */
/*   Address */
#define NET_SECURITY_IDENTTYPE_ADDRESS         1
/*   Prefix */
#define NET_SECURITY_IDENTTYPE_PREFIX          2
/*   FQDN */
#define NET_SECURITY_IDENTTYPE_FQDN            3
/*   USERFQDN */
#define NET_SECURITY_IDENTTYPE_USERFQDN        4
/*   Maximum value */
#define NET_SECURITY_IDENTTYPE_MAX             NET_SECURITY_IDENTTYPE_USERFQDN

/* Maximum buffer needed to hold an identity string */
#define NET_SECURITY_IDENTSTR_MAX              1024

/* Receive identity types (for net_security_receiveident() et al.): */
/*   Receive local identity information */
#define NET_SECURITY_RECEIVEIDENT_LOCAL        1
/*   Receive remote identity information */
#define NET_SECURITY_RECEIVEIDENT_REMOTE       2

/* Maximum buffer needed to hold a request in string form */
#define NET_SECURITY_REQUESTSTR_MAX           (NET_SECURITY_OPERATION_MAX*10+1)

#if !KERNEL
int net_security_setrequest(int s, int number, void *request, int requestlen);
int net_security_getrequest(int s, int number, void **request, int *requestlen);
int net_security_activerequests(int s, fd_set *which);
int net_security_inputrequests(int s, fd_set *which);
int net_security_setnum(int s, int num);
int net_security_getnum(int s, int *num);
int net_security_receivenum(int s, int onoff);
int net_security_cmsgtonum(void *cmsg, int cmsglen, int *num);
int net_security_numtocmsg(int num, void *cmsg, int *cmsglen, int maxlen);
int net_security_setlocalident(int s, int identtype, char *identstr);
int net_security_getlocalident(int s, int *identtype, char **identstr);
int net_security_setremoteident(int s, int identtype, char *identstr);
int net_security_getremoteident(int s, int *identtype, char **identstr);
int net_security_receiveident(int s, int which);
int net_security_cmsgtolocalident(void *cmsg, int cmsglen, int *identtype, char *identstr, int maxlen);
int net_security_localidenttocmsg(int identtype, char *identstr, void *cmsg, int *cmsglen, int maxlen);
int net_security_cmsgtoremoteident(void *cmsg, int cmsglen, int *identtype, char *identstr, int maxlen);
int net_security_remoteidenttocmsg(int identtype, char *identstr, void *cmsg, int *cmsglen, int maxlen);
int net_security_strtorequest(char *str, void **request, int *requestlen);
int net_security_requesttostr(void *request, int requestlen, char *str, int maxlen);
#endif /* !KERNEL */

#ifndef EPOLICY
#define EPOLICY 255 /* XXX! XXX! */
#endif /* EPOLICY */

#ifdef __NET_SECURITY_LLIF
#ifndef SO_SECURITY_MULTIPLEX
#define SO_SECURITY_MULTIPLEX 28
#endif /* SO_SECURITY_MULTIPLEX */

struct __net_security_arg {
  int op;
  int n;
  union {
    size_t val;
    size_t *p;
  } len;
  union {
    int val;
    void *p;
  } data;
};

#define __NET_SECURITY_ARG_REQUEST            1
#define __NET_SECURITY_ARG_REQUESTNUM         2
#define __NET_SECURITY_ARG_REQUESTACTIVE      3
#define __NET_SECURITY_ARG_REQUESTINPUTMASK   4
#define __NET_SECURITY_ARG_IDENTITYLOCAL      5
#define __NET_SECURITY_ARG_IDENTITYREMOTE     6
#define __NET_SECURITY_ARG_POLICYMATCHRULES   7
#define __NET_SECURITY_ARG_POLICYHINTRULES    8
#define __NET_SECURITY_ARG_POLICYREQUESTS     9

#define __NET_SECURITY_ARG_MAXLEN             (NET_SECURITY_OPERATION_MAX*sizeof(struct net_security_operation))
#endif /* __NET_SECURITY_LLIF */

#if KERNEL
#include <net/security_var.h>
#endif /* KERNEL */

#endif /* _NET_SECURITY_H */
